/*
 *  bignum.h
 *  GlowWorm
 *
 *  Created by Curtis Jones on 2006.06.28.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __BIGNUM_H__
#define __BIGNUM_H__

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <sys/types.h>

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define BIGNUM_BASE_TYPE		u_int32_t

//#define BIGNUM_SOFB_08		1
//#define BIGNUM_SOFB_16		2
#define BIGNUM_SOFB_32			3
//#define BIGNUM_SOFB_64		4
//#define BIGNUM_SOFB_128		5

#define BIGNUM_POSITIVE			1
#define BIGNUM_NEGATIVE			2

#define BIGNUM_MAX_ORDERS		100000

#define BIGNUM_MAX(a,b) ((a) > (b) ? (a) : (b))
#define BIGNUM_MIN(a,b) ((a) < (b) ? (a) : (b))

struct bignum
{
	u_int8_t						sign;					// BIGNUM_POSITIVE or BIGNUM_NEGATIVE
	u_int64_t						oofm;					// orders of magnitude
	u_int64_t						sofb;					// size of base ... sizeof(u_int8_t)
	u_int64_t						size;					// storage capacity for number ... numb[size]
	u_int64_t						base;					// base of storage system
	BIGNUM_BASE_TYPE	* numb;					// number storage
};
typedef struct bignum bignum_t;





bignum_t * bignum_alloc ();
void bignum_free (bignum_t *num);
void bignum_init ();

bignum_t * bignum_rnd_mag (bignum_t *num, u_int64_t size);
bignum_t * bignum_rnd_max (bignum_t *num, bignum_t *max);

bignum_t * bignum_set (bignum_t *, u_int64_t val);
bignum_t * bignum_set_ui8 (bignum_t *, u_int64_t cnt, BIGNUM_BASE_TYPE val[]);
bignum_t * bignum_set_bn (bignum_t *, bignum_t *);
bignum_t * bignum_set_str (bignum_t *, unsigned int radix, const char *);
void bignum_print (bignum_t *);
int bignum_cstr (bignum_t *, int len, char *str);
int bignum_cstr_base (bignum_t *, int str_len, char *str, int base);
char * bignum_cstr_base2 (bignum_t *, int precision, int base);
unsigned char * bignum_getbytes(bignum_t *, unsigned char *);
bignum_t * bignum_setbytes (bignum_t *, unsigned char *, int len);

u_int32_t bignum_get_uint32 (bignum_t *num);

bignum_t * bignum_rshift (bignum_t *num, u_int64_t cnt);
int bignum_isbitset (bignum_t *num, u_int64_t bit);
u_int64_t bignum_lowest_set_bit (bignum_t *num);
u_int64_t bignum_bitlen (bignum_t *num);
u_int64_t bignum_bytelen (bignum_t *num);

bignum_t * bignum_add (bignum_t *sum, bignum_t *num1, bignum_t *num2);
bignum_t * bignum_sub (bignum_t *dif, bignum_t *min, bignum_t *sub);
bignum_t * bignum_mul (bignum_t *prod, bignum_t *fac1, bignum_t *fac2);
bignum_t * bignum_div_aux (bignum_t *quot, bignum_t *rmdr, bignum_t *divd, bignum_t *divs);
bignum_t * bignum_div (bignum_t *quot, bignum_t *rmdr, bignum_t *divd, bignum_t *divs);
bignum_t * bignum_mod (bignum_t *rmdr, bignum_t *divd, bignum_t *divs);
bignum_t * bignum_modpow (bignum_t *res, bignum_t *num, bignum_t *exp, bignum_t *mod);
bignum_t * bignum_modinv (bignum_t *res_d, bignum_t *res_x, bignum_t *res_y, bignum_t *num, bignum_t *mod);
bignum_t * bignum_gcd (bignum_t *res, bignum_t *num1, bignum_t *num2);
bignum_t * bignum_pow (bignum_t *Y, bignum_t *num, bignum_t *exp);

bignum_t * bignum_factorial (bignum_t *total, bignum_t *num);
bignum_t * bignum_factorial_poor (bignum_t *total, bignum_t *num);

inline int bignum_cmp (bignum_t *num1, bignum_t *num2);
inline int bignum_eq (bignum_t *num1, bignum_t *num2);
inline int bignum_gt (bignum_t *num1, bignum_t *num2);
inline int bignum_lt (bignum_t *num1, bignum_t *num2);
inline int bignum_iszero (bignum_t *num);

inline BIGNUM_BASE_TYPE __bignum_get (bignum_t *num, u_int64_t pos);
inline void __bignum_set (bignum_t *num, u_int64_t pos, BIGNUM_BASE_TYPE val);

void __bignum_grow (bignum_t *num, u_int64_t size);
void __bignum_zero (bignum_t *num);
void __bignum_trim (bignum_t *num);

bignum_t * bignum_zero ();
bignum_t * bignum_one ();
bignum_t * bignum_two ();
bignum_t * bignum_three ();
bignum_t * bignum_four ();
bignum_t * bignum_five ();
bignum_t * bignum_six ();
bignum_t * bignum_seven ();
bignum_t * bignum_eight ();
bignum_t * bignum_nine ();
bignum_t * bignum_ten ();
bignum_t * bignum_eleven ();
bignum_t * bignum_twelve ();
bignum_t * bignum_thirteen ();
bignum_t * bignum_fourteen ();
bignum_t * bignum_fifteen ();
bignum_t * bignum_sixteen ();
bignum_t * bignum_seventeen ();
bignum_t * bignum_eighteen ();
bignum_t * bignum_nineteen ();
bignum_t * bignum_twenty ();
bignum_t * bignum_thirty_two ();
bignum_t * bignum_one_hundred ();

#endif /* __BIGNUM_H__ */
