/*
 *  bignum_add.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2007.08.05.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

// Knuth 4.3.1 - Algorithm A
//
bignum_t *
bignum_add (bignum_t *sum, bignum_t *num1, bignum_t *num2)
{
	u_int64_t j = 0;					// iterates through the digits of the number
	u_int64_t k = 0;					// tracks carries 
	u_int64_t n = 0;					// max magnitudes of num1, num2
	u_int64_t b = sum->base;	// radix base
	
	__bignum_zero(sum);
	
	// if exactly one of the two numbers is negative,
	// perform subtraction after temporarily setting
	// that negative number to positive.
	//
	if (num1->sign == BIGNUM_NEGATIVE && num2->sign == BIGNUM_POSITIVE) {
		num1->sign = BIGNUM_POSITIVE;
		bignum_sub(sum, num1, num2);
		sum->sign = sum->sign == BIGNUM_POSITIVE ? BIGNUM_NEGATIVE : BIGNUM_POSITIVE;
		num1->sign = BIGNUM_NEGATIVE;
		return sum;
	}
	else if (num1->sign == BIGNUM_POSITIVE && num2->sign == BIGNUM_NEGATIVE) {
		num2->sign = BIGNUM_POSITIVE;
		bignum_sub(sum, num1, num2);
		sum->sign = sum->sign == BIGNUM_POSITIVE ? BIGNUM_NEGATIVE : BIGNUM_POSITIVE;
		num2->sign = BIGNUM_NEGATIVE;
		return sum;
	}
	
	// n represents the magnitude of the larger of the
	// two numbers. this is the number of times that we
	// must iterate in the next step.
	//
	n = BIGNUM_MAX(num1->oofm, num2->oofm);

	// STEP A2
	//
	// sum_sub_j = (num1_sub_j + num2_sub_j + k) mod b
	// k = floor_of(num1_sub_j + num2_sub_j + k) / b
	//
	do {
		u_int64_t x, y;
		
		x = (u_int64_t)__bignum_get(num1,j) + (u_int64_t)__bignum_get(num2,j) + k;
		k = x / b;
		y = x - (k * b);
		
		__bignum_set(sum, j, y);
	}
	while (++j < n);
	
	// STEP A3
	//
	// if k != 0
	//   sum_sub_j = k (if k != 0)
	//
	if (k != 0) {
		__bignum_set(sum, j, k);
		sum->oofm = j + 1;
	}
	else
		sum->oofm = j;
	
	// if both of the numbers are negative, then the sum is
	// also negative.
	//
	if (num1->sign == BIGNUM_NEGATIVE && num2->sign == BIGNUM_NEGATIVE)
		sum->sign = BIGNUM_NEGATIVE;
	
	return sum;
}
