/*
 *  bignum_factorial.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2008.03.01.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

/**
 * Calculate num factorial into total and return total.
 *
 */
bignum_t *
bignum_factorial (bignum_t *total, bignum_t *num)
{
	return bignum_factorial_poor(total, num);
}

/**
 * Poor man's factorial which uses multiplication.
 *
 */
bignum_t *
bignum_factorial_poor (bignum_t *total, bignum_t *num)
{
	bignum_t * tmp1 = bignum_alloc();
	bignum_t * tmp2 = bignum_alloc();
	
	__bignum_zero(total);
	
	bignum_set_bn(total, num);
	bignum_sub(tmp1, total, bignum_one());
	
	// while tmp1 > 1
	//   tmp2 = total * tmp1
	//   total = tmp2
	//   tmp2 = tmp1 - 1
	//   tmp1 = tpm2;
	//
	while (bignum_gt(tmp1,bignum_one())) {
		bignum_mul(tmp2, total, tmp1);
		bignum_set_bn(total, tmp2);
		bignum_sub(tmp2, tmp1, bignum_one());
		bignum_set_bn(tmp1, tmp2);
	}
	
	if (num->sign == BIGNUM_NEGATIVE)
		total->sign = BIGNUM_NEGATIVE;
	
	bignum_free(tmp1);
	bignum_free(tmp2);
	
	return total;
}
