/*
 *  bignum_mod.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2007.08.05.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

bignum_t *
bignum_mod (bignum_t *rmdr, bignum_t *divd, bignum_t *divs)
{
	bignum_div(NULL, rmdr, divd, divs);
	
	return rmdr;
}

#define BIGNUM_TIMEDIF(tv1,tv2)	(	\
	tv1.tv_usec > tv2.tv_usec ?	\
		(((tv2.tv_sec - tv1.tv_sec) - 1.0) + ((tv2.tv_usec + (1000000 - tv1.tv_usec)) / 1000000.0))	\
	:	\
		(((tv2.tv_sec - tv1.tv_sec)      ) + ((tv2.tv_usec - tv1.tv_usec) / 1000000.0))	\
	)

bignum_t *
bignum_modpow (bignum_t *res, bignum_t *num, bignum_t *exp, bignum_t *mod)
{
	bignum_t *tmp  = bignum_alloc();
	bignum_t *_exp = bignum_alloc();
	bignum_t *_num = bignum_alloc();
	
	bignum_set(res,  1);				// result = 1
	bignum_set_bn(_exp, exp);
	bignum_set_bn(_num, num);
	
	while ( bignum_gt(_exp,bignum_zero()) ) {
		if (bignum_isbitset(_exp,1)) {
			bignum_mul(tmp, res, _num);
			bignum_mod(res, tmp, mod);
		}
		
		bignum_rshift(_exp, 1);
		bignum_pow(tmp, _num, bignum_two());
		bignum_mod(_num, tmp, mod);
	}
	
	bignum_free( tmp  );
	bignum_free( _exp );
	bignum_free( _num );
	
	return res;
}

// a = num
// b = mod
// y = res
// a >= b
//
bignum_t *
bignum_modinv (bignum_t *res_d, bignum_t *res_x, bignum_t *res_y, bignum_t *num, bignum_t *mod)
{
	bignum_t * a    = bignum_alloc();
	bignum_t * b    = bignum_alloc();
	bignum_t * x    = bignum_alloc();
	bignum_t * x1   = bignum_alloc();
	bignum_t * x2   = bignum_alloc();
	bignum_t * r    = bignum_alloc();
	bignum_t * y    = bignum_alloc();
	bignum_t * y1   = bignum_alloc();
	bignum_t * y2   = bignum_alloc();
	bignum_t * q    = bignum_alloc();
	bignum_t * d    = bignum_alloc();
	bignum_t * tmp1 = bignum_alloc();
	
	bignum_set_bn(a, num);
	bignum_set_bn(b, mod);
	
	if (res_d != NULL) __bignum_zero( res_d );
	if (res_x != NULL) __bignum_zero( res_x );
	if (res_y != NULL) __bignum_zero( res_y );
	
	// if b == 0, then y = 0; nothing more to do
	//
	if (bignum_iszero(b))
		goto bail;
	
	bignum_set(x2, 1);
	bignum_set(x1, 0);
	bignum_set(y2, 0);
	bignum_set(y1, 1);
	
	while (bignum_gt(b,bignum_zero()))
	{
		bignum_div(q, NULL, a, b);					// q = a / b
		
		bignum_mul(tmp1, q, b);							// tmp1 = q * b
		bignum_sub(r, a, tmp1);							// r = a - tmp1
		
		bignum_mul(tmp1, q, x1);						// tmp1 = q * x1
		bignum_sub(x, x2, tmp1);						// x = x2 - tmp1
		
		bignum_mul(tmp1, q, y1);						// tmp1 = q * y1
		bignum_sub(y, y2, tmp1);						// y = y2 - tmp1
		
		bignum_set_bn(a,  b );
		bignum_set_bn(b,  r );
		bignum_set_bn(x2, x1);
		bignum_set_bn(x1, x );
		bignum_set_bn(y2, y1);
		bignum_set_bn(y1, y );
	}
	
	bignum_set_bn(d, a );
	bignum_set_bn(x, x2);
	bignum_set_bn(y, y2);
	
bail:
	if (res_d != NULL) bignum_set_bn(res_d, a);
	if (res_x != NULL) bignum_set_bn(res_x, x2);
	if (res_y != NULL) bignum_set_bn(res_y, y2);
	
	bignum_free( a    );
	bignum_free( b    );
	bignum_free( x    );
	bignum_free( x1   );
	bignum_free( x2   );
	bignum_free( r    );
	bignum_free( y    );
	bignum_free( y1   );
	bignum_free( y2   );
	bignum_free( q    );
	bignum_free( d    );
	bignum_free( tmp1 );
	
	return res_y;
}
