/*
 *  bignum_mul.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2007.08.05.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

// Knuth 4.3.1 - Algorithm M
//
// factor1 x factor2 = product
//
bignum_t *
bignum_mul (bignum_t *prod, bignum_t *fac1, bignum_t *fac2)
{
	u_int64_t j = 0;
	u_int64_t t = 0;
	u_int64_t b = prod->base;
	u_int64_t m = fac1->oofm;
	
	// if either number is zero, the product is 
	// also zero. return.
	//
	if (bignum_iszero(fac1) || bignum_iszero(fac2)) {
		bignum_set(prod, 0);
		return prod;
	}
	
	// STEP M1
	//
	__bignum_zero(prod);
	
	// STEP M2 - M6
	//
	// i = 0
	// k = 0
	//
	// t = fac1_sub_i x fac2_sub_j + prod_sub_i+j + k
	// prod_sub_i+j = t mod b
	// k = floor_of(t / b)
	//
	// prod_sub_j+m = k
	//
	do {
		u_int64_t i = 0;
		u_int64_t k = 0;
		u_int64_t y = 0;
		
		if (0 != __bignum_get(fac2,j)) {
			do {
				t = (u_int64_t)__bignum_get(fac1,i) * (u_int64_t)__bignum_get(fac2,j) + (u_int64_t)__bignum_get(prod,i+j) + k;
				k = t / b;
				y = t - (k * b);
				
				__bignum_set(prod, i+j, y);
				
				if (y != 0 && i+j >= prod->oofm)
					prod->oofm = i + j + 1;
			}
			while (++i < fac1->oofm);
		}
		
		__bignum_set(prod, j+m, k);
		
		if (k != 0 && j+m >= prod->oofm)
			prod->oofm = j + m + 1;
	}
	while (++j < fac2->oofm);
	
	if (fac1->sign == BIGNUM_NEGATIVE && fac2->sign == BIGNUM_NEGATIVE)
		prod->sign = BIGNUM_POSITIVE;
	else if (fac1->sign == BIGNUM_NEGATIVE || fac2->sign == BIGNUM_NEGATIVE)
		prod->sign = BIGNUM_NEGATIVE;
	
	return prod;
}
