/*
 *  bignum_rnd.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2007.08.05.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

/**
 * Sets num to a random value that is of the order of size. Returns a reference
 * to num.
 *
 */
bignum_t *
bignum_rnd_mag (bignum_t *num, u_int64_t size)
{
	u_int64_t i = 0;
	
	for (; i < size; ++i)
		__bignum_set(num, i, random());
	
	num->oofm = size;
	
	return num;
}

/**
 * Sets num to a random value that is less than max but usually of the same
 * magnitude as max. Returns a reference to num.
 *
 */
bignum_t *
bignum_rnd_max (bignum_t *num, bignum_t *max)
{
	u_int64_t i = 0;
	
	__bignum_trim( max );
	
	if (0 != max->oofm) {
		for (; i < max->oofm; ++i) {
			__bignum_set(num, i, random());			
			if (num->numb[i] != 0)
				num->oofm = i + 1;
		}
		
		__bignum_set(num, i-1, random() % max->numb[i-1]);
		
		if (num->numb[0] != 0)
			num->oofm = BIGNUM_MAX(num->oofm, 1);
	}
	
	return num;
}
