/*
 *  bignum_set.c
 *  BigMath
 *
 *  Created by Curtis Jones on 2007.08.05.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "bignum.h"

char BIGNUM_BASE_CHAR_INV[256] =
{
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 000 - 015
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 016 - 031
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 032 - 047
	000,   1,   2,   3,   4,   5,   6,   7,   8,   9, 000, 000, 000, 000, 000, 000,			// 048 - 063
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 064 - 079
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 080 - 095
	000,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,			// 096 - 111
	 25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35, 000, 000, 000, 000, 000,			// 112 - 127
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 128 - 143
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 144 - 159
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 160 - 175
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 176 - 191
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 192 - 207
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 208 - 223
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 224 - 239
	000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000, 000,			// 239 - 255
};

/**
 * Sets the value of num to val. Returns a reference to num.
 *
 */
bignum_t *
bignum_set (bignum_t *num, u_int64_t val)
{
	bzero(num->numb, num->size * num->sofb);
	
	BIGNUM_BASE_TYPE m1 = (val      ) & 0xFFFFFFFFull;
	BIGNUM_BASE_TYPE m2 = (val >> 32) & 0xFFFFFFFFull;
	
	if (m2 != 0) {
		__bignum_set(num, 0, m1);
		__bignum_set(num, 1, m2);
		num->oofm = 2;
	}
	else if (m1 != 0) {
		__bignum_set(num, 0, m1);
		num->oofm = 1;
	}
	else
		num->oofm = 0;
	
	if (val >= 0)
		num->sign = BIGNUM_POSITIVE;
	else
		num->sign = BIGNUM_NEGATIVE;
	
	return num;
}

/**
 * Sets the value of num to the value indicated in str, by converting the
 * string representation into a numeric value. This function expects str to be
 * null terminated.
 *
 * Maximum supported radix (at the time of writing, anyway) is thirty-six, 
 * which must be represented by 0-9, a-z (note that it is lower-case letters).
 *
 */
bignum_t *
bignum_set_str (bignum_t *num, unsigned int radix, const char *str)
{
	int digits;
	const char * str_ptr;
	
	bignum_t * ret = NULL;
	bignum_t * multiplier = NULL;
	bignum_t * sum = NULL;
	bignum_t * radixx = NULL;
	bignum_t * digit = NULL;
	bignum_t * tmp1 = NULL;
	
	if (radix > 36)
		return NULL;
	
	if (str == NULL || 0 == (digits = strlen(str)))
		return NULL;
	
	if (NULL == (multiplier = bignum_alloc()))
		goto finish;
	
	if (NULL == (sum = bignum_alloc()))
		goto finish;
	
	if (NULL == (radixx = bignum_alloc()))
		goto finish;
	
	if (NULL == (digit = bignum_alloc()))
		goto finish;
	
	if (NULL == (tmp1 = bignum_alloc()))
		goto finish;
	
	bignum_set(radixx, radix);
	bignum_set(multiplier, 1);
	
	str_ptr = str + digits - 1;
	
	// set our multiplier equal to 1
	// for each digit in str (reversed)
	//   get decimal value
	//   multiply by multiplier
	//   add to sum
	//   multiply multiplier by radix
	//
	do {
		bignum_set(digit, BIGNUM_BASE_CHAR_INV[*str_ptr]);
		bignum_mul(tmp1, digit, multiplier);
		bignum_add(sum, num, tmp1);
		bignum_set_bn(num, sum);
		bignum_mul(tmp1, multiplier, radixx);
		bignum_set_bn(multiplier, tmp1);
	}
	while (--str_ptr >= str);
	
	ret = num;
	
finish:
	if (multiplier != NULL)
		bignum_free(multiplier);
		
	if (sum != NULL)
		bignum_free(sum);
	
	if (radixx != NULL)
		bignum_free(radixx);
	
	if (digit != NULL)
		bignum_free(digit);
	
	if (tmp1 != NULL)
		bignum_free(tmp1);
	
	return ret;
}

/**
 * Sets the value of num to val. val is an array of BIGNUM_BASE_TYPEs of size
 * cnt. val must be of the correct byte order (little endian?). Returns a 
 * reference to num.
 *
 */
bignum_t *
bignum_set_ui8 (bignum_t *num, u_int64_t cnt, BIGNUM_BASE_TYPE val[])
{
	u_int64_t i = cnt - 1;
	u_int64_t j = 0;
	
	for (; j < cnt; --i, ++j) {
		__bignum_set(num, j, val[i]);
		
		if (num->numb[j] != 0)
			num->oofm = j + 1;
	}
	
	return num;
}

/**
 * Sets the value of num1 to num2. Returns a reference to num1.
 *
 */
bignum_t *
bignum_set_bn (bignum_t *num1, bignum_t *num2)
{
	if (num1 == NULL)
		num1 = bignum_alloc();
	
	if (num1->size <= num2->oofm)
		__bignum_grow(num1, num2->oofm);
	
	memcpy(num1->numb, num2->numb, num2->oofm * num2->sofb);
	
	num1->oofm = num2->oofm;
	num1->sign = num2->sign;
	
	return num1;
}
